#' Import Rdpack for Rd macros
#'
#' Ensures Rd macros from Rdpack are available without triggering an unused
#' import note during R CMD check.
#' @name healthmarkers-rdpack-import
#' @keywords internal
#' @import Rdpack
NULL
