% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz-rdpack.R
\name{healthmarkers-rdpack-import}
\alias{healthmarkers-rdpack-import}
\title{Import Rdpack for Rd macros}
\description{
Ensures Rd macros from Rdpack are available without triggering an unused
import note during R CMD check.
}
\keyword{internal}
