% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cvd_risk.R
\name{cvd_risk_scorescvd}
\alias{cvd_risk_scorescvd}
\title{RiskScorescvd calculator}
\usage{
cvd_risk_scorescvd(data, ...)
}
\arguments{
\item{data}{Data required by \code{RiskScorescvd::calc_scores()}.}

\item{...}{Passed to \code{RiskScorescvd::calc_scores()}.}
}
\value{
Object returned by \code{RiskScorescvd::calc_scores()}.
}
\description{
Passthrough to \code{RiskScorescvd::calc_scores()} with graceful fallback to NA.
}
\examples{
\donttest{
  if (requireNamespace("RiskScorescvd", quietly = TRUE)) {
    df <- data.frame(
      Age = 55, Sex = 0, Smoking_status = 1,
      systolic.bp = 140, Total_cholesterol = 5.5,
      HDL.cholesterol = 1.3
    )
    cvd_risk_scorescvd(df)
  }
}
}
